%THIS FUNCTION ASSEMBLES THE PUZZLE PIECES. A PIECE IS SELECTED AS THE SEED
%AND ALL ITS MATCHING PIECES ARE ASSEMBLED TO IT. SEQUENTIALLY, THE SEEDS
%OF THE ORIGINAL PIECE WILL BECOME THE NEW SEED. THE SEED METHOD IS AN
%EFFORT TO AVOID THE PROBLEM OF SUB PUZZLE ASSEMBLY.


function [out1,out2,out3,out4] = AssemblyRevise(puzzle,sol,moved,clusters)

%COMPILING SOLVED PIECES MATRIX FOR PARTIAL SOLUTION ROTATION
matchMatrix = [];
cnt = 0;
for h=1:length(puzzle)
    if (isempty(sol(h).edge)==0)
        for i=1:length(sol(h).edge(:,1))
            if (sol(h).edge{i,1}(1,1)~=0)
                if sol(h).edge{i,1}(1,1)>sol(h).edge{i,2}(1,1)
                    continue
                end
                cnt                = cnt+1;
                matchMatrix(cnt,1) = sol(h).edge{i,1}(1,1);
                matchMatrix(cnt,2) = sol(h).edge{i,1}(1,2);
                matchMatrix(cnt,3) = sol(h).edge{i,2}(1,1);
                matchMatrix(cnt,4) = sol(h).edge{i,2}(1,2);
            end
        end
    end
end

%DETERMINE SEEDS OF EACH PIECE, SEEDS:{PIECE,SEEDS, LENGTH}
if isempty(matchMatrix)==0
    seeds = cell(length(puzzle),3);
    for h=1:length(puzzle)
        seeds{h,1} = h;
        cnt        = 0;
        %FINDING MATCHES
        for i=1:length(matchMatrix(:,1))
            if (matchMatrix(i,1)==h)
                cnt               = cnt+1;
                seeds{h,2}(1,cnt) = matchMatrix(i,3);
            elseif (matchMatrix(i,3)==h)
                cnt               = cnt+1;
                seeds{h,2}(1,cnt) = matchMatrix(i,1);
            end
        end
        %DETERMINING NUMBER OF MATCHES
        if (isempty(seeds{h,2}))
            seeds{h,3} = 0;
        else
            seeds{h,3} = length(seeds{h,2});
        end
    end
    
    %REMOVING MATCHLESS PIECES FROM BEING SEEDS
    usedSeeds = zeros(length(puzzle),1);
    for h=1:length(puzzle)
        if (seeds{h,3}==0)
            usedSeeds(h) = 1;
        end
    end
    
    %DETERMINING ORDER OF SEEDS
    num = 0;
    while (sum(usedSeeds)~=length(puzzle))
        %DETERMINING SEED BY PIECE WITH MOST MATCHES (HIGHEST matchNumber)
        matchNumber = 0;
        for i=1:length(puzzle)
            if (usedSeeds(i,1)==0 && seeds{i,3}>=matchNumber)
                matchNumber    = seeds{i,3};
                seed           = i;
            end
        end
        
        %ADDING SEED TO THE SEED ORDER
        if num==0
            num               = num+1;
            seedOrder(num,1)  = seed;
            usedSeeds(seed)   = 1;
        else
            num              = length(seedOrder)+1;
            seedOrder(num,1) = i;
            usedSeeds(seed)  = 1;
        end
        
        %ADDING THE SEED'S MATCHES TO THE SEED ORDER
        for i=1:seeds{seed,3}
            if (usedSeeds(seeds{seed,2}(1,i),1)==0)
                cnt                           = cnt+1;
                seedOrder(cnt,1)              = seeds{seed,2}(1,i);
                usedSeeds(seeds{seed,2}(1,i)) = 1;
            end
        end
    end
    
    %DETERMING LENGTH OF clusters
    if isempty(clusters)
        clusterCount = 0;
    else
        clusterCount = length(clusters);
    end
    
    %ASSEMBLYING SEED AND ITS MATCHES
    for h=1:length(seedOrder(:,1))
        %COMPILE ASSEMBLE MATRIX
        p1             = seedOrder(h,1);
        assemblyMatrix = [];
        cnt            = 0;
        for i=1:length(matchMatrix(:,1))
            if (matchMatrix(i,1)==p1 || matchMatrix(i,3)==p1)
                cnt                   = cnt+1;
                assemblyMatrix(cnt,:) = matchMatrix(i,:);
            end
        end
        if isempty(assemblyMatrix)
            continue
        end
        for i=1:length(assemblyMatrix(:,1))
            %DETERMINING PIECE 2
            if (assemblyMatrix(i,1)==p1)
                p2 = assemblyMatrix(i,3);
                e1 = assemblyMatrix(i,2);
                e2 = assemblyMatrix(i,4);
            else
                p2 = assemblyMatrix(i,1);
                e1 = assemblyMatrix(i,4);
                e2 = assemblyMatrix(i,2);
            end
            
            %IF NEITHER PIECE HAS BEEN MOVED
            if moved(p1)==0 && moved(p2)==0
                %fprintf('(%d,%d) and (%d,%d)\n',p1,e1,p2,e2);
                rigidTransformation = PutTogetherAdvanced(puzzle(p1).edge{e1,4},puzzle(p2).edge{e2,4},puzzle(p1).piece,puzzle(p2).piece,puzzle(p1).edge{e1,5},puzzle(p2).edge{e2,5});
                move                = p2;
                update              = 0;
                
            elseif moved(p1)==1 && moved(p2)==0
                %fprintf('(%d,%d) and (%d,%d)\n',p1,e1,p2,e2);
                rigidTransformation = PutTogetherAdvanced(puzzle(p1).edge{e1,4},puzzle(p2).edge{e2,4},puzzle(p1).piece,puzzle(p2).piece,puzzle(p1).edge{e1,5},puzzle(p2).edge{e2,5});
                move                = p2;
                
                %UPDATING clusters
                for J=1:length(clusters)
                    if ismember(p1, clusters{J,1})==1
                        mergeClusterIndex = J;
                        update            = 1;
                        break
                    end
                end
            elseif moved(p1)==0 && moved(p2)==1
                %fprintf('(%d,%d) and (%d,%d)\n',p1,e1,p2,e2);
                rigidTransformation = PutTogetherAdvanced(puzzle(p2).edge{e2,4},puzzle(p1).edge{e1,4},puzzle(p2).piece,puzzle(p1).piece,puzzle(p2).edge{e2,5},puzzle(p1).edge{e1,5});
                move                = p1;
                
                %UPDATING clusters
                for J=1:length(clusters)
                    if ismember(p2,clusters{J,1})==1
                        mergeClusterIndex = J;
                        update            = 1;
                        break
                    end
                end
            elseif moved(p1)==1 && moved(p2)==1
                %DETERMINING PIECES' CLUSTER
                sameCluster = 0;
                cluster1    = [];
                cluster2    = [];
                
                for J=1:length(clusters(:,1))
                    if ismember(p1,clusters{J,1}) && ismember(p2,clusters{J,1})
                        sameCluster   = 1;
                        break
                    elseif ismember(p1,clusters{J,1})
                        cluster1      = clusters{J,1};
                        cluster1Index = J;
                    elseif ismember(p2,clusters{J,1})
                        cluster2      = clusters{J,1};
                        cluster2Index = J;
                    end
                end
                
                %TESTING IF THE PIECES ARE IN THE SAME CLUSTER
                if sameCluster==1
                    continue
                end
                
                %CALCULATING RIGID TRANSFORMATION MATRIX
                if length(cluster1)>length(cluster2)
                    %fprintf('(%d,%d) and (%d,%d):\n',p1,e1,p2,e2);
                    rigidTransformation = PutTogetherAdvanced(puzzle(p1).edge{e1,4},puzzle(p2).edge{e2,4},puzzle(p1).piece,puzzle(p2).piece,puzzle(p1).edge{e1,5},puzzle(p2).edge{e2,5});
                    deleteCluster       = unique(cluster2);
                    keepClusterIndex    = cluster1Index;
                    deleteClusterIndex  = cluster2Index;
                else
                    %fprintf('(%d,%d) and (%d,%d)\n',p1,e1,p2,e2);
                    rigidTransformation = PutTogetherAdvanced(puzzle(p2).edge{e2,4},puzzle(p1).edge{e1,4},puzzle(p2).piece,puzzle(p1).piece,puzzle(p2).edge{e2,5},puzzle(p1).edge{e1,5});
                    deleteCluster       = unique(cluster1);
                    keepClusterIndex    = cluster2Index;
                    deleteClusterIndex  = cluster1Index;
                end
                
                %TRANSFORMING PIECES
                if (rigidTransformation~='w')
                    rotation    = rigidTransformation(1:3,1:3);
                    translation = rigidTransformation(1:3,4);
                    
                    for j=1:length(deleteCluster)
                        for k=1:length(puzzle(deleteCluster(j)).piece(:,1))
                            puzzle(deleteCluster(j)).piece(k,:) = (rotation*(puzzle(deleteCluster(j)).piece(k,:))'+translation)';
                        end
                        for k=1:length(puzzle(deleteCluster(j)).edge(:,1))
                            for l=1:length(puzzle(deleteCluster(j)).edge{k,1}(:,1))
                                puzzle(deleteCluster(j)).edge{k,1}(l,:) = (rotation*(puzzle(deleteCluster(j)).edge{k,1}(l,:))'+translation)';
                            end
                            for l=1:length(puzzle(deleteCluster(j)).edge{k,4}(:,1))
                                puzzle(deleteCluster(j)).edge{k,4}(l,:) = (rotation*(puzzle(deleteCluster(j)).edge{k,4}(l,:))'+translation)';
                            end
                        end
                    end
                end
                
                %COMBINING CLUSTERS
                clusters{keepClusterIndex,1} = union(clusters{deleteClusterIndex,1}, clusters{keepClusterIndex,1});
                
                %UPDATING CLUSTERS
                clusters{deleteClusterIndex,1} = [];
                temp                           = [];
                count                          = 0;
                for j=1:length(clusters)
                    if isempty(clusters)==0
                        count = count+1;
                        temp{count,1}=unique(clusters{j,1});
                    end
                end
                clusters = [];
                clusters = temp;
                continue
            end
            
            %ACQUIRING ROTATION MATRIX AND SHIFT, ASSEMBLES PIECE 2 TO PIECE 1
            if rigidTransformation~='w'
                rotation    = rigidTransformation(1:3,1:3);
                translation = rigidTransformation(1:3,4);
                moved(p1)   = 1;
                moved(p2)   = 1;
                
                %UPDATING ROTATED PIECE
                for j=1:length(puzzle(move).piece(:,1))
                    puzzle(move).piece(j,:) = (rotation*(puzzle(move).piece(j,:))'+translation)';
                end
                for j=1:length(puzzle(move).edge(:,1))
                    for l=1:length(puzzle(move).edge{j,1}(:,1))
                        puzzle(move).edge{j,1}(l,:) = (rotation*(puzzle(move).edge{j,1}(l,:))'+translation)';
                    end
                    for l=1:length(puzzle(move).edge{j,4}(:,1))
                        puzzle(move).edge{j,4}(l,:) = (rotation*(puzzle(move).edge{j,4}(l,:))'+translation)';
                    end
                end
                
                %UPDATING MOVED PIECE TO CLUSTER
                if update==1
                    clusters{mergeClusterIndex,1}(end+1) = move;
                elseif update==0
                    clusterCount                = clusterCount+1;
                    clusters{clusterCount,1}(1) = p1;
                    clusters{clusterCount,1}(2) = p2;
                end
            elseif rigidTransformation=='w'%ELIMINATING THE BAD MATCH
                indexCnt = 0;
                while indexCnt<length(puzzle(p1).edge{3,1}(:,1))
                    indexCnt = indexCnt+1;
                    if puzzle(p1).edge{e1,3}(indexCnt,2)==p2 && puzzle(p1).edge{e1,3}(indexCnt,3)==e2
                        puzzle(p1).edge{e1,3}(indexCnt,1) = 10;
                        puzzle(p1).edge{e1,3}             = sortrows(puzzle(p1).edge{e1,3},1);
                        sol(p1).edge{e1,1}                = zeros(1,2);
                        sol(p1).edge{e1,2}                = zeros(1,2);
                        for t=1:length(sol(p1).match)
                            if p2==sol(p1).match(t)
                                sol(p1).match(t) = [];
                                break
                            end
                        end
                        break
                    end
                end
                indexCnt = 0;
                while indexCnt<length(puzzle(p2).edge{e2,3}(:,1))
                    indexCnt = indexCnt+1;
                    if puzzle(p2).edge{e2,3}(indexCnt,2)==p1 && puzzle(p2).edge{e2,3}(indexCnt,3)==e1
                        puzzle(p2).edge{e2,3}(indexCnt,1) = 10;
                        puzzle(p2).edge{e2,3}             = sortrows(puzzle(p2).edge{e2,3},1);
                        sol(p2).edge{e2,1}                = zeros(1,2);
                        sol(p2).edge{e2,2}                = zeros(1,2);
                        for t=1:length(sol(p2).match)
                            if p1==sol(p2).match(t)
                                sol(p2).match(t) = [];
                                break
                            end
                        end
                        break
                    end
                end
            end
        end
    end
    
    %UPDATING CLUSTERS
    for h=1:length(clusters)
        clusters{h,1} = unique(clusters{h,1});
    end
end

%OUTPUTS
out1 = puzzle;
out2 = sol;
out3 = moved;
out4 = clusters;
end

%% PROBLEMS
% 1. MARK PIECES IN SOLUTION STRUCTURE THAT HAVE ALREADY BEEN MOVED

%GO BACK AND DELETE BAD MATCHED FROM THE SKEWMATRIX