%FUNCTION: This function generates a curvy shaped puzzle edge given four
    %points from the Voronoi diagram
    
%INPUT: Vertives on the Voronoi and should be given as (x,y) points
    %start-edge point and initial point of the edge 
    %leftcent- mid point of voronoi and left of start 
    %rightcent- mid point of voronoi and right of start 
    %fin- edge point and final point of the edge 

%OUTPUT: Discrete Bezier curve that forms our puzzle edge


function edge=CurvyEdge(start, leftcent, rightcent, fin)

%PARAMETRIZING DIAGONAL TO EXTRACT POTENTIAL CONTROL POINTS
dx=1/300; t=0:dx:1; r=rand;
%START TO LEFTCENT (SL)
slx=(1-t)*(start(1))+t*leftcent(1);
sly=(1-t)*(start(2))+t*leftcent(2);
slz=(1-t)*(start(3))+t*leftcent(3);
%START TO RIGHTCENT (SR)
srx=(1-t)*(start(1))+t*rightcent(1);
sry=(1-t)*(start(2))+t*rightcent(2);
srz=(1-t)*(start(3))+t*rightcent(3);
%FIN TO LEFTCENT (FL)
flx=(1-t)*(fin(1))+t*leftcent(1);
fly=(1-t)*(fin(2))+t*leftcent(2);
flz=(1-t)*(fin(3))+t*leftcent(3);
%FIN TO RIGHTCENT (FR)
frx=(1-t)*(fin(1))+t*rightcent(1);
fry=(1-t)*(fin(2))+t*rightcent(2);
frz=(1-t)*(fin(2))+t*rightcent(3);

%COMPILING POTENTIAL PATHS FOR CURVE
path1(:,1)=slx; path1(:,2)=sly; path1(:,3)=slz; path1(:,4)=frx; path1(:,5)=fry; path1(:,6)=frz;
path2(:,1)=srx; path2(:,2)=sry; path2(:,3)=srz; path2(:,4)=flx; path2(:,5)=fly; path2(:,6)=flz;
path3(:,1)=srx; path3(:,2)=sry; path3(:,3)=srz; path3(:,4)=frx; path3(:,5)=fry; path3(:,6)=frz;
path4(:,1)=slx; path4(:,2)=sly; path4(:,3)=slz; path4(:,4)=flx; path4(:,5)=fly; path4(:,6)=flz;

%DETERMING PATH OF CONTROL POINTS
if r<.25
    path=path1;
elseif r>0.25 && r<0.5
    path=path2;
elseif r>0.5 && r<0.75
    path=path3;
else
    path=path4;
end

%GENERATES TWO RANDOM CONTROL POINTS
len=length(path(:,1)); bezx=zeros(len,1); bezy=zeros(len,1); bezz=zeros(len,1);
rlow=randi([2,len-1]); rhigh=randi([2,len-1]);

con1x=path(rlow,1); con1y=path(rlow,2); con1z=path(rlow,3);
con2x=path(rhigh,1); con2y=path(rhigh,2); con2z=path(rhigh,3);

for i=1:(1+(1/dx))
    bezx(i,1)=start(1)*(1-t(i))^3+3*t(i)*con1x*(1-t(i))^2+3*(t(i)^2)*con2x*(1-t(i))+fin(1)*t(i)^3;
    bezy(i,1)=start(2)*(1-t(i))^3+3*t(i)*con1y*(1-t(i))^2+3*(t(i)^2)*con2y*(1-t(i))+fin(2)*t(i)^3;
    bezz(i,1)=start(3)*(1-t(i))^3+3*t(i)*con1z*(1-t(i))^2+3*(t(i)^2)*con2z*(1-t(i))+fin(3)*t(i)^3;
end
edge(:,1)=bezx; edge(:,2)=bezy; edge(:,3)=bezz;
for p=1:length(edge(:,1))
    [ph, thet, rh]=cart2sph(edge(p,1),edge(p,2),edge(p,3));
    rho(p,1)=rh; theta(p,1)=ph; phi(p,1)=thet;
    if rho(p,1)~=1
        rho(p,1)=1;
    end
    [x,y,z]=sph2cart(theta(p,1),phi(p,1),rho(p,1));
    edge(p,3)=x; edge(p,2)=y; edge(p,3)=z;
end


end