%INPUT: Entire puzzle in the form of a cell array 
%FUNCTION: Partitions the signature wrt the kappa axis.
%OUTPUT: Structure with column containing the partitioned signature, 
%   which corresponds to another column containing the edges.

function out = KappaSegmentation(x)

%ACQUIRING SIGNATURE CURVE DATA
for i=1:length(x)
    
%     figure(1); hold on
%     plot3(x{i,1}(:,1),x{i,1}(:,2),x{i,1}(:,3));
    
    t            = [];
    t            = Noise(x{i,1});
    result       = Smoother(t,0.026); %0.015 for 0.005 
    
    %plot3(result(:,1),result(:,2),result(:,3));
    
%     figure(2);
%     plot3(result(:,1),result(:,2),result(:,3),'r');
    
    
    temp         = compsig(result')';
    shift        = rand;
    trans(1:3,1) = 10*shift; 
    
    for j=1:length(result(:,1))
       insy(j,:)    = (result(j,:)'+trans)'; 
    end
    puzzle(i).piece = insy(:,1:3);
    puzzle(i).sig   = temp(:,1:3);    
end

%DELETING NaN VALUES
for h=1:length(x);
    cnt   = 0;
    piece = [];
    sig   = [];
    for i=1:length(puzzle(h).sig(:,1))
        if (isfinite(puzzle(h).sig(i,1)) && ...
                isfinite(puzzle(h).sig(i,2)) && ...
                isfinite(puzzle(h).sig(i,3)) && ...
                isfinite(puzzle(h).piece(i,1)) && ...
                isfinite(puzzle(h).piece(i,2)) && ...
                isfinite(puzzle(h).piece(i,3)))
            
            cnt          = cnt+1;
            piece(cnt,:) = puzzle(h).piece(i,:);
            sig(cnt,:)   = puzzle(h).sig(i,:);
        end
    end
    puzzle(h).sig = [];     puzzle(h).piece = [];
    puzzle(h).sig = sig;    puzzle(h).piece = piece;   
end

%PARTITION
for h=1:length(x);
    index = 1;
    num   = 1;
    cnt   = 1;
    while (abs(puzzle(h).sig(index,1))<0.8)  %1
        index = index+1;
    end
    puzzle(h).edge{num,1}(1,:) = puzzle(h).piece(index,:);
    puzzle(h).edge{num,2}(1,:) = puzzle(h).sig(index,:);
    for i=index+1:length(puzzle(h).sig(:,1))
        if abs(puzzle(h).sig(i,1))>0.8    %1
            cnt = cnt+1;
            puzzle(h).edge{num,1}(cnt,:) = puzzle(h).piece(i,:);
            puzzle(h).edge{num,2}(cnt,:) = puzzle(h).sig(i,:);
            puzzle(h).edge{num,4}(cnt,1) = i;
        else
            num = num+1;
            cnt = 1;
            puzzle(h).edge{num,1}(cnt,:) = puzzle(h).piece(i,:);
            puzzle(h).edge{num,2}(cnt,:) = puzzle(h).sig(i,:);
            puzzle(h).edge{num,4}(cnt,1) = i;
        end
    end 
end

%SIPHONING SEGMENTS 
for h=1:length(x)
    for i=length(puzzle(h).edge(:,1)):-1:1
        if isempty(puzzle(h).edge{i,1})
            continue
        end
        if length(puzzle(h).edge{i,2})<35    %50
            puzzle(h).edge{i,1} = []; 
            puzzle(h).edge{i,2} = []; 
            puzzle(h).edge{i,4} = [];
        end
    end
end

%RESTRUCTURING SEGMENTS
for h=1:length(x)
    cnt=0; here=[];
    for i=1:length(puzzle(h).edge(:,1))
        if isempty(puzzle(h).edge{i,1})==0
            cnt         = cnt+1;
            here{cnt,1} = puzzle(h).edge{i,1};
            here{cnt,2} = puzzle(h).edge{i,2};
            here{cnt,4} = puzzle(h).edge{i,4};
        end
    end
    puzzle(h).edge = []; puzzle(h).edge = here;
end

%SMOOTHING SEGMENTS
for h=1:length(x);
   for i=1:length(puzzle(h).edge(:,1))
       indices             = [];
       
%        figure(1); hold on
%        plot3(puzzle(h).edge{i,2}(:,1),puzzle(h).edge{i,2}(:,2),puzzle(h).edge{i,2}(:,3));
       puzzle(h).edge{i,5} = puzzle(h).edge{i,1};
       puzzle(h).edge{i,1} = SmootherSegments(puzzle(h).edge{i,1},.00000001); 
       puzzle(h).edge{i,2} = SmootherSegments(puzzle(h).edge{i,2},.00000001);  
       indices             = puzzle(h).edge{i,4};
       puzzle(h).edge{i,4} = [];

       % plot3(puzzle(h).edge{i,2}(:,1),puzzle(h).edge{i,2}(:,2),puzzle(h).edge{i,2}(:,3),'.r');

       
       %PRECEDING 50 POINTS
       for j=35:-1:1
           if indices(1)-j>0
               puzzle(h).edge{i,4}(end+1,:) = puzzle(h).piece(indices(1)-j,:);
           else
               puzzle(h).edge{i,4}(end+1,:) = puzzle(h).piece(indices(1)-j+length(puzzle(h).piece(:,1)),:);           
          end
       end
       
       %INSERTING EDGE
       for j=1:length(puzzle(h).edge{j,1}(:,1))
          puzzle(h).edge{i,4}(end+1,:) = puzzle(h).edge{i,1}(j,:); 
       end
       
       %SUCCEDING 50 POINTS
       for j=1:35
           if indices(end)+j<=length(puzzle(h).piece(:,1))
               puzzle(h).edge{i,4}(end+1,:) = puzzle(h).piece(indices(end)+j,:);
           else
               puzzle(h).edge{i,4}(end+1,:) = puzzle(h).piece(indices(end)+j-length(puzzle(h).piece(:,1)),:);
           end
       end
       puzzle(h).edge{i,4} = SmootherSegments(puzzle(h).edge{i,4},.00000001);
%        figure(1); hold on
%        plot3(test1(3:end-3,1),test1(3:end-3,2),test1(3:end-3,3),'r');
%        plot3(test1(3:end-3,1),test1(3:end-3,2),test1(3:end-3,3),'.r');
%        plot3(puzzle(h).edge{i,2}(:,1),puzzle(h).edge{i,2}(:,2),puzzle(h).edge{i,2}(:,3),'k')
%        plot3(puzzle(h).edge{i,2}(:,1),puzzle(h).edge{i,2}(:,2),puzzle(h).edge{i,2}(:,3),'.k')
%        plot3(test1(1:3,1),test1(1:3,2),test1(1:3,3),'g','LineWidth',2);
   end
end

out = puzzle;

end

%%