%THIS FUNCTION CALCULATES THE SIMILARITY BETWEEN TWO SEGMENTS OF A
%SIGNATURE CURVES, WHICH IS THE INPUT. THE OUTPUT IS A SKEWNESS VALUE WHICH
%IS GREATER THAN ZERO AND THE SIMILARITY MEASURE.

function out = compareEgg3DEXP(sink,source)
% figure(1); hold on
% plot3(sink(:,1),sink(:,2),sink(:,3));
% plot3(sink(:,1),sink(:,2),sink(:,3),'.b');
% plot3(source(:,1),source(:,2),source(:,3),'r');
% plot3(source(:,1),source(:,2),source(:,3),'.r');
% plot3(sink(1,1),sink(1,2),sink(1,3),'.g');
% plot3(source(1,1),source(1,2),source(1,3),'.g');

%SIMILARITY CALCULATION 
for t=1:8  
    %DETERMINING INITIAL AND FINAL POINTS
    if t==5
        for i=1:length(sink(:,1))
            sink(i,2) = -sink(i,2);
        end
    end
    if t==1 || t==5
        seg1 = sink;
        seg2 = source;    
    elseif t==2 || t==6
        seg1 = sink;
        seg2 = (fliplr(source'))';
    elseif t==3 || t==7
        seg1 = fliplr(sink')';
        seg2 = source;    
    else
        seg1 = fliplr(sink')';
        seg2 = (fliplr(source'))';
    end
    
%     if t==1
%         figure(2); hold on
%         plot3(seg1(:,1),seg1(:,2),seg1(:,3));
%         plot3(seg1(:,1),seg1(:,2),seg1(:,3),'.b');
%         plot3(seg2(:,1),seg2(:,2),seg2(:,3),'k');
%         plot3(seg2(:,1),seg2(:,2),seg2(:,3),'.k');
%         
%         plot3(seg1(1,1),seg1(1,2),seg1(1,3),'.g');
%         plot3(seg2(1,1),seg2(1,2),seg2(1,3),'.g');
%         plot3(seg1(50,1),seg1(50,2),seg1(50,3),'.c');
%         plot3(seg2(50,1),seg2(50,2),seg2(50,3),'.c');
%         plot3(seg1(end,1),seg1(end,2),seg1(end,3),'.r');
%         plot3(seg2(end,1),seg2(end,2),seg2(end,3),'.r');
%     end
    
    %CONVERSION TO SPHERICAL COORDINATES
    polar1=zeros(length(seg1),3); polar2=zeros(length(seg1),3); 
    for i=1:length(seg1)
        [theta,phi,rho] = cart2sph(seg1(i,1),seg1(i,2),seg1(i,3));
        if phi<0
            phi = -phi+pi/2;
        else
            phi = pi/2-phi;
        end
        while theta<0
            theta = theta+2*pi;
        end
        polar1(i,1)     = rho;
        polar1(i,2)     = phi;
        polar1(i,3)     = theta;
    end
    for i=1:length(seg2)
        [theta,phi,rho] = cart2sph(seg2(i,1),seg2(i,2),seg2(i,3));
        if phi<0
            phi = -phi+pi/2;
        else
            phi = pi/2-phi;
        end
        while theta<0
            theta = theta+2*pi;
        end
        polar2(i,1)     = rho;
        polar2(i,2)     = phi;
        polar2(i,3)     = theta;
    end      
    
%     %APPLY PROCRUSTES
%     matPolar1(:,1) = 1:length(polar1(:,1));
%     matPolar1(:,2) = polar1(:,1);
%     matPolar1(:,3) = 0;
%     
%     matPolar2(:,1) = 1:length(polar2(:,1));
%     matPolar2(:,2) = polar2(:,1);
%     matPolar2(:,3) = 0;
%     
%     matRot         = estimateRigidTransform2D(matPolar1',matPolar2');
%     rotationMatrix = matRot(1:3,1:3);
%     translation    = matRot(1:3,4);
%     
%     figure(1); hold on
%     plot(1:length(polar1(:,1)),polar1(:,1),'.k');
%     plot(1:length(polar1(:,1)),polar2(:,1),'.r');
%     
%     for i=1:length(polar2(:,1))
%         matPolar2(i,:)=(rotationMatrix*(matPolar2(i,:))'+translation)';
%     end
%     
%     plot(matPolar2(:,1),matPolar2(:,2),'.g');
    
    
    %CUMULATIVE CALCULATION
    for i=2:length(polar1)
        polar1(i,1) = polar1(i,1)+polar1(i-1,1);
        polar1(i,2) = polar1(i,2)+polar1(i-1,2);
        polar1(i,3) = polar1(i,3)+polar1(i-1,3);
    end
    for i=2:length(seg2)
        polar2(i,1) = polar2(i,1)+polar2(i-1,1);
        polar2(i,2) = polar2(i,2)+polar2(i-1,2);
        polar2(i,3) = polar2(i,3)+polar2(i-1,3);
    end
    
    %CUMULATIVE CALCULATION
    cnt            = 0;
    mean           = round(length(polar1(:,1))/2);
    cumPolar1(1,:) = polar1(mean,:);
    cumPolar2(1,:) = polar2(mean,:);
    
    while cnt<length(polar1(:,1))
        cnt = cnt+2;
        if cnt+mean<length(polar1(:,1))
            cumPolar1(cnt+1,:) = polar1(mean+cnt,:);
            cumPolar2(cnt+1,:) = polar2(mean+cnt,:);
        end
        if mean-cnt>0
            cumPolar1(cnt+2,:) = polar1(mean-cnt,:);
            cumPolar2(cnt+2,:) = polar2(mean-cnt,:);
        end
    end
    
    polar2    = -polar2;
    cumPolar2 = -cumPolar2;
    
    %COMPILING ORDERED CUMULATIVE VECTORS
    distVector  = polar1(:,1);   distVector(length(polar1(:,1))+1:length(polar1(:,1))+length(polar2(:,1)))  = polar2(:,1);
    thetaVector = polar1(:,2);   thetaVector(length(polar1(:,2))+1:length(polar1(:,2))+length(polar2(:,2))) = polar2(:,2);
    phiVector   = polar1(:,3);   phiVector(length(polar1(:,3))+1:length(polar1(:,3))+length(polar2(:,3)))   = polar2(:,3);
        
    %CALCULATING ORDERED SKEW
    skewTheta(t) = abs(((1/length(thetaVector))*sum((thetaVector).^3))/((1/length(thetaVector))*sum((thetaVector).^2))^(1.5));
    skewPhi(t)   = abs(((1/length(phiVector))*sum((phiVector).^3))/((1/length(phiVector))*sum((phiVector).^2))^(1.5));
    skewDist(t)  = abs(((1/length(distVector))*sum((distVector).^3))/((1/length(distVector))*sum((distVector).^2))^(1.5));
    temp(t)      = skewDist(t)+skewPhi(t)+skewTheta(t);
    
    
    %COMPILING UNORDERED CUMULATIVE VECTORS
    cumDistVector  = cumPolar1(:,1);   cumDistVector(length(cumPolar1(:,1))+1:length(cumPolar1(:,1))+length(cumPolar2(:,1)))  = cumPolar2(:,1);
    cumThetaVector = cumPolar1(:,2);   cumThetaVector(length(cumPolar1(:,2))+1:length(cumPolar1(:,2))+length(cumPolar2(:,2))) = cumPolar2(:,2);
    cumPhiVector   = cumPolar1(:,3);   cumPhiVector(length(cumPolar1(:,3))+1:length(cumPolar1(:,3))+length(cumPolar2(:,3)))   = cumPolar2(:,3);
    
    %CALCULATING UNORDERED SKEW
    cumSkewTheta(t) = abs(((1/length(cumThetaVector))*sum((cumThetaVector).^3))/((1/length(cumThetaVector))*sum((cumThetaVector).^2))^(1.5));
    cumSkewPhi(t)   = abs(((1/length(cumPhiVector))*sum((cumPhiVector).^3))/((1/length(cumPhiVector))*sum((cumPhiVector).^2))^(1.5));
    cumSkewDist(t)  = abs(((1/length(cumDistVector))*sum((cumDistVector).^3))/((1/length(cumDistVector))*sum((cumDistVector).^2))^(1.5));
    cumTemp(t)      = cumSkewDist(t)+cumSkewPhi(t)+cumSkewTheta(t);
    %fprintf('THE SKEWS ARE %f AND %f AND %f\n',skew,skewphi,skewtheta);
end

% fprintf('The standard cumulative skew is: %f\n',min(temp));
% fprintf('The min of this skew vector is: %f\n',min(skewTheta)+min(skewPhi)+min(skewDist));
% 
% fprintf('The expirement cumulative skew is: %f\n',min(cumTemp));
% fprintf('The min of this skew vector is: %f\n',min(cumSkewTheta)+min(cumSkewPhi)+min(cumSkewDist));
% t=1;

out = min(cumSkewTheta)+min(cumSkewPhi)+min(cumSkewDist);
end

%%
